package com.caimei.module.product.service.impl;

import com.caimei.module.base.entity.bo.PageVo;
import com.caimei.module.base.entity.po.SeconHandProduct;
import com.caimei.module.base.entity.po.Smalltype;
import com.caimei.module.base.entity.po.Tinytype;
import com.caimei.module.base.entity.vo.*;
import com.caimei.module.product.dao.ProductModuleDao;
import com.caimei.module.product.service.ProductModuleService;
import com.caimei.module.product.util.ProductUtils;
import com.github.pagehelper.PageHelper;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.stream.Collectors;

/**
 * Description
 *
 * @author : Charles
 * @date : 2020/3/25
 */
@Service
public class ProductModuleServiceImpl implements ProductModuleService {
    @Resource
    private ProductModuleDao productModuleDao;

    /**
     * 获取分类列表
     * typeSort 分类类型：1产品，2仪器
     * source 请求来源：www,crm
     */
    @Override
    public List<BigtypeVo> getClassify(String domain, String typeSort, String source) {
        List<BigtypeVo> bigTypeList = productModuleDao.getBigType(typeSort,source);
        for (BigtypeVo bigType : bigTypeList) {
            String caiMeiImage = ProductUtils.getImageURL("caiMeiImage", null, 0, domain);
            bigType.setWwwIcon(StringUtils.isEmpty(bigType.getWwwIcon())?caiMeiImage:bigType.getWwwIcon());
            bigType.setCrmIcon(StringUtils.isEmpty(bigType.getCrmIcon())?caiMeiImage:bigType.getCrmIcon());
            List<SmalltypeVo> smalltypeList = productModuleDao.getSmallType(bigType.getBigTypeID(), source);
            for (SmalltypeVo smalltype : smalltypeList) {
                smalltype.setWwwIcon(StringUtils.isEmpty(smalltype.getWwwIcon())?caiMeiImage:smalltype.getWwwIcon());
                smalltype.setCrmIcon(StringUtils.isEmpty(smalltype.getCrmIcon())?caiMeiImage:smalltype.getCrmIcon());
                List<TinytypeVo> tinytypeList = productModuleDao.getTinytype(smalltype.getSmallTypeID(), source);
                if (!CollectionUtils.isEmpty(tinytypeList)) {
                    for (TinytypeVo tinyType : tinytypeList) {
                        tinyType.setWwwIcon(StringUtils.isEmpty(tinyType.getWwwIcon())?caiMeiImage:tinyType.getWwwIcon());
                        tinyType.setCrmIcon(StringUtils.isEmpty(tinyType.getCrmIcon())?caiMeiImage:tinyType.getCrmIcon());
                    }
                    smalltype.setTinytypeList(tinytypeList);
                }
            }
            if (!CollectionUtils.isEmpty(smalltypeList)) {
                bigType.setSmalltypeList(smalltypeList);
            }
        }
        return bigTypeList;
    }

    /**
     * 根据分类ID获取二三级分类
     * bigTypeID 一级分类ID
     * source 请求来源：www,crm
     * @return
     */
    @Override
    public BigtypeVo getClassifyByTypeId(String domain, Integer typeId, Integer idType, String source) {
        Integer bigTypeID = 0;
        if (idType == 1) {
            bigTypeID = typeId;
        } else if (idType == 2 ) {
            bigTypeID = productModuleDao.getBigTypeIdBySmallTypeId(typeId);
        } else if (idType == 3 ) {
            bigTypeID = productModuleDao.getBigTypeIdByTinyTypeId(typeId);
        }
        BigtypeVo bigType = productModuleDao.getBigTypeByBigTypeId(bigTypeID);
        bigType.setIsChecked(false);
        List<SmalltypeVo> smallTypeList = productModuleDao.getSmallType(bigTypeID,source);
        for (SmalltypeVo smallType : smallTypeList) {
            String caiMeiImage = ProductUtils.getImageURL("caiMeiImage", null, 0, domain);
            smallType.setWwwIcon(StringUtils.isEmpty(smallType.getWwwIcon())?caiMeiImage:smallType.getWwwIcon());
            smallType.setCrmIcon(StringUtils.isEmpty(smallType.getCrmIcon())?caiMeiImage:smallType.getCrmIcon());
            smallType.setIsChecked(false);
            List<TinytypeVo> tinytypeList = productModuleDao.getTinytype(smallType.getSmallTypeID(), source);
            if (!CollectionUtils.isEmpty(tinytypeList)) {
                for (TinytypeVo tinyType : tinytypeList) {
                    tinyType.setWwwIcon(StringUtils.isEmpty(tinyType.getWwwIcon())?caiMeiImage:tinyType.getWwwIcon());
                    tinyType.setCrmIcon(StringUtils.isEmpty(tinyType.getCrmIcon())?caiMeiImage:tinyType.getCrmIcon());
                    tinyType.setIsChecked(false);
                }
                smallType.setTinytypeList(tinytypeList);
            }
            if (!CollectionUtils.isEmpty(smallTypeList)) {
                bigType.setSmalltypeList(smallTypeList);
            }
        }
        return bigType;
    }

    /**
     * 根据一级分类ID获取二三级分类
     * bigTypeID 一级分类ID
     * source 请求来源：www,crm
     */
    @Override
    public List<SmalltypeVo> getBigTypeClassify(String domain, Integer bigTypeID, String source) {
        List<SmalltypeVo> smallTypeList = productModuleDao.getSmallType(bigTypeID,source);
        for (SmalltypeVo smallType : smallTypeList) {
            String caiMeiImage = ProductUtils.getImageURL("caiMeiImage", null, 0, domain);
            smallType.setWwwIcon(StringUtils.isEmpty(smallType.getWwwIcon())?caiMeiImage:smallType.getWwwIcon());
            smallType.setCrmIcon(StringUtils.isEmpty(smallType.getCrmIcon())?caiMeiImage:smallType.getCrmIcon());
            List<TinytypeVo> tinytypeList = productModuleDao.getTinytype(smallType.getSmallTypeID(), source);
            if (!CollectionUtils.isEmpty(tinytypeList)) {
                for (TinytypeVo tinyType : tinytypeList) {
                    tinyType.setWwwIcon(StringUtils.isEmpty(tinyType.getWwwIcon())?caiMeiImage:tinyType.getWwwIcon());
                    tinyType.setCrmIcon(StringUtils.isEmpty(tinyType.getCrmIcon())?caiMeiImage:tinyType.getCrmIcon());
                }
                smallType.setTinytypeList(tinytypeList);
            }
        }
        return smallTypeList;
    }

    /**
     * 根据一级/二级/三级分类获取商品
     *
     * @param tinyTypeID
     * @param sortType
     * @param pageNum
     * @param pageSize
     */
    @Override
    public PageVo<ProductVo> getProductsByTypeId(Integer bigTypeID, Integer smallTypeID, Integer tinyTypeID, Integer sortType, String source, Integer pageNum, Integer pageSize) {
        // sortType (3:价格升序, 4:价格降序, 7:人气, 8:销量)
        pageNum = null != pageNum ? pageNum : 1;
        pageSize = null != pageSize ? pageSize : 10;
        PageHelper.startPage(pageNum, pageSize);
        sortType = null != sortType ? sortType : 8;
        List<Integer> bigTypeIDList = new ArrayList<>();
        List<Integer> smallTypeIDList = new ArrayList<>();
        List<Integer> tinyTypeIDList = new ArrayList<>();
        if (bigTypeID != null) {
            bigTypeIDList.add(bigTypeID);
        }
        if (smallTypeID != null) {
            smallTypeIDList.add(smallTypeID);
        }
        if (tinyTypeID != null) {
            tinyTypeIDList.add(tinyTypeID);
        }
        if (tinyTypeIDList.size() > 0) {
            bigTypeIDList.clear();
            smallTypeIDList.clear();
        } else if (smallTypeIDList.size() > 0) {
            bigTypeIDList.clear();
        }
        //当二级和三级分类id列表都为空时，根据一级分类id获取二级分类id列表
        if (smallTypeIDList.size() == 0 && tinyTypeIDList.size() == 0) {
            List<SmalltypeVo> smalltypeVoList = productModuleDao.getSmallType(bigTypeID, source);
            if (smalltypeVoList.size() > 0) {
                bigTypeIDList.clear();
                smallTypeIDList.addAll(smalltypeVoList.stream().map(Smalltype::getSmallTypeID).collect(Collectors.toList())) ;
            }
        }
        //当二级分类id列表不为空且三级分类id列表为空时，获取三级分类id列表
        if (smallTypeIDList.size()>0 && tinyTypeIDList.size() == 0) {
            Iterator<Integer> iterator = smallTypeIDList.iterator();
            while (iterator.hasNext()) {
                Integer smallId = iterator.next();
                List<TinytypeVo> tinytypeVoList = productModuleDao.getTinytype(smallId, source);
                if (tinytypeVoList.size() > 0) {
                    iterator.remove();
                    tinyTypeIDList.addAll(tinytypeVoList.stream().map(Tinytype::getTinyTypeID).collect(Collectors.toList()));
                }
            }
        }
        List<ProductVo> productList = productModuleDao.getProductsByTypeId(bigTypeIDList, smallTypeIDList, tinyTypeIDList, sortType);
        return new PageVo<>(productList);
    }

    /**
     * 获取模块商品列表
     *
     * @param preferredFlag 新品上线(001) 优惠商品(010) 常用商品(100)，三者同时存在111
     * @param userId
     * @param pageNum
     * @param pageSize
     */
    @Override
    public PageVo<ProductVo> getPreferred(Integer preferredFlag, Integer userId, Integer pageNum, Integer pageSize) {
        pageNum = null != pageNum ? pageNum : 1;
        pageSize = null != pageSize ? pageSize : 10;
        PageHelper.startPage(pageNum, pageSize);
        List<ProductVo> productList = productModuleDao.getProductsByPreferredFlag(preferredFlag);
        return new PageVo<>(productList);
    }

    /**
     * 根据关键词获取商品
     *
     * @param searchWord
     * @param userId
     * @param pageNum
     * @param pageSize
     */
    @Override
    public PageVo<ProductVo> searchProduct(String searchWord, Integer userId, Integer pageNum, Integer pageSize) {
        pageNum = null != pageNum ? pageNum : 1;
        pageSize = null != pageSize ? pageSize : 10;
        // 登录状态下，查询历史记录是否存在
        if (null != userId) {
            List<SearchHistoryVo> historyList = productModuleDao.getSearchHistoryList(userId);
            //保存并删除大于10条的历史记录
            SearchHistoryVo historyRecord = new SearchHistoryVo();
            historyRecord.setUserId(userId.longValue());
            historyRecord.setSearchDate(new Date());
            historyRecord.setSearchWord(searchWord);
            historyRecord.setDelFlag("0");
            if (historyList != null) {
                Long recordId = productModuleDao.getSearchHistoryIdByWord(searchWord);
                if (recordId != null && recordId > 0L) {
                    historyRecord.setId(recordId);
                    productModuleDao.updateSearchHistoryById(historyRecord);
                } else {
                    productModuleDao.insertSearchHistory(historyRecord);
                }
                if (historyList.size() >= 10) {
                    productModuleDao.deleteSearchHistoryByUserId(userId);
                }
            } else {
                productModuleDao.insertSearchHistory(historyRecord);
            }
        }
        PageHelper.startPage(pageNum, pageSize);
        List<ProductVo> productList = productModuleDao.searchProduct(searchWord);
        return new PageVo<>(productList);
    }


    @Override
    public List<ProductVo> getRecommended(Integer productID) {
        ProductVo product = productModuleDao.findProductById(productID);
        List<ProductVo> productList = null;
        if ("1".equals(product.getRecommendType())) {
            List<Integer> productIDs = productModuleDao.getRecommendIds(productID);
            if (productIDs != null && productIDs.size() > 0) {
                productList = productModuleDao.getRecommendByIds(productIDs);
            }
        } else {
            productList = productModuleDao.getDeFaultRecommend(product.getTinyTypeID());
        }
        return productList;
    }

    @Override
    public AddressVo getAddressInfo(Integer townId){
        return productModuleDao.getAddressInfo(townId);
    }

    @Override
    public void saveSencondHandProduct(SeconHandProduct seconHandProduct) {
        productModuleDao.saveSencondHandProduct(seconHandProduct);
    }

    /**
     * 商品搜索历史记录
     */
    @Override
    public List<SearchHistoryVo> getSearchHistory(Integer userId) {
        return productModuleDao.getSearchHistoryList(userId);
    }

    /**
     * 根据用户ID删除历史记录
     *
     * @param userId
     */
    @Override
    public void deleteAllSearchHistory(Integer userId) {
        productModuleDao.deleteAllSearchHistory(userId);
    }

    /**
     * 商品详情页
     *
     * @param productID
     */
    @Override
    public ProductVo getProductDetails(Integer productID, String domain) {
        ProductVo product = productModuleDao.findProductUnlimited(productID);
        if (product == null) {
            //商品不存在
            product = new ProductVo();
            product.setValidFlag("0");
            return product;
        } else if (!"2".equals(product.getProductCategory()) && "2".equals(product.getValidFlag()) && (product.getStock()== null || product.getStock() == 0)) {
            //已上架但库存为0的正常商品，设置为已售罄商品
            product.setValidFlag("4");
        }
        String[] split = null;
        //商品标签
        if (product.getTags() != null) {
            String tags = product.getTags();
            tags = tags.replace(" ", ",").replace("、", ",").replace("，", ",");
            if (tags.contains(",")) {
                split = tags.split(",");
                List<String> list = new ArrayList();
                for (String s : split) {
                    if (s != null && !s.equals("")) {
                        list.add(s);
                    }
                }
                split = list.toArray(new String[list.size()]);
            }
        }
        product.setTagsList(split);
        product.setMainImage(ProductUtils.getImageURL("product", product.getMainImage(), 0, domain));
        //商品图片
        List<ProductImageVo> imageList = productModuleDao.findProductImage(product.getProductID());
        for (ProductImageVo image : imageList) {
            String imageURL = ProductUtils.getImageURL("product", image.getImage(), 0, domain);
            image.setImage(imageURL);
        }
        product.setImageList(imageList);
        //商品详情
        ProductDetailVo productDetail = productModuleDao.findProductDetailById(productID);
        product.setProductDetail(productDetail);
        //供应商信息
        ShopVo shop = productModuleDao.findProductShopById(product.getShopID());
        Integer normalNum = productModuleDao.findProductNormalNumById(product.getShopID());
        shop.setNormalNum(normalNum == null ? 0 : normalNum);
        if (shop.getBusinessScope() != null && shop.getBusinessScope().contains("/")) {
            String[] businessScopeArray = shop.getBusinessScope().split("/");
            shop.setBusinessScopeArray(businessScopeArray);
        }
        shop.setLogo(ProductUtils.getImageURL("shopLogo", shop.getLogo(), 0, domain));
        shop.setBusinessLicenseImage(ProductUtils.getImageURL("shopLogo", shop.getBusinessLicenseImage(), 0, domain));
        product.setShop(shop);
        //价格等级
        String priceGrade = ProductUtils.getPriceGrade(product.getPrice1());
        product.setPrice1Grade(priceGrade);
        //商品分类
        String typeName = productModuleDao.findTypeName(product.getBigTypeID(),product.getSmallTypeID(), product.getTinyTypeID());
        product.setTypeName(typeName);
        //品牌名称
        BrandVo brand = productModuleDao.getBrand(product.getBrandID());
        if (null != brand) {
            product.setBrandName(brand.getName());
        }
        return product;
    }

    /**
     * 设置商品展示价格
     *
     * @param product
     * @param userId
     */
    @Override
    public ProductVo getShowPrice(ProductVo product, Integer userId) {
        product.setRetailPrice(new BigDecimal(product.getPrice1()));
        //是否启用阶梯价格
        if (null != userId && product.getLadderPriceFlag().equals("1")) {
            LadderPriceVo ladderPrice = productModuleDao.findLowerLadderPrice(product.getProductID());
            product.setRetailPrice(ladderPrice.getBuyPrice());
        }
        return product;
    }

    /**
     * 获取阶梯价格
     *
     * @param productID
     */
    @Override
    public List<LadderPriceVo> findLadderPrice(Integer productID) {
        return productModuleDao.findLadderPrice(productID);
    }

    /**
     * 获取一条最低阶梯价格
     *
     * @param productID
     */
    @Override
    public LadderPriceVo findLowerLadderPrice(Integer productID) {
        return productModuleDao.findLowerLadderPrice(productID);
    }


}
